﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Data;

namespace MediaInfo
{
  [TemplatePart(Name = "DescriptionPanel", Type = typeof(FrameworkElement))]
  [TemplateVisualState(Name = "Normal", GroupName = "CommonStates")]
  [TemplateVisualState(Name = "MouseOver", GroupName = "CommonStates")]
  [TemplateVisualState(Name = "DescriptionNormal",
    GroupName = "DescriptionStates")]
  [TemplateVisualState(Name = "DescriptionExpanded",
    GroupName = "DescriptionStates")]
  public class MediaInfoDisplay : Control
  {
    private const int SHORT_LENGTH = 30;

    public MediaInfoDisplay()
    {
      DefaultStyleKey = typeof(MediaInfoDisplay);
    }

    // Short description DP
    public string ShortDescription
    {
      get { return (string) GetValue(ShortDescriptionProperty); }
      set { SetValue(ShortDescriptionProperty, value); }
    }

    public static readonly DependencyProperty ShortDescriptionProperty
      = DependencyProperty.Register("ShortDescription",
      typeof(string), typeof(MediaInfoDisplay),
      new PropertyMetadata(null));

    // Long description DP
    private static void OnLongChanged(DependencyObject d,
      DependencyPropertyChangedEventArgs args)
    {
      MediaInfoDisplay sender = d as MediaInfoDisplay;

      sender.ShortDescription
        = (sender.LongDescription.Length > SHORT_LENGTH)
        ? sender.LongDescription.Substring(0, SHORT_LENGTH - 3) + "..."
        : sender.LongDescription;
    }

    public string LongDescription
    {
      get { return (string) GetValue(LongDescriptionProperty); }
      set { SetValue(LongDescriptionProperty, value); }
    }

    public static readonly DependencyProperty LongDescriptionProperty
      = DependencyProperty.Register("LongDescription",
      typeof(string), typeof(MediaInfoDisplay),
      new PropertyMetadata(new PropertyChangedCallback(OnLongChanged)));

    // Media info DP
    private static void OnMediaChanged(DependencyObject d,
      DependencyPropertyChangedEventArgs args)
    {
      MediaInfoDisplay sender = d as MediaInfoDisplay;

      Binding binding = new Binding();
      binding.Source = args.NewValue as Media;
      binding.Path = new PropertyPath("Description");
      sender.SetBinding(LongDescriptionProperty, binding);
    }

    public Media MediaInfo
    {
      get { return (Media) GetValue(MediaInfoProperty); }
      set { SetValue(MediaInfoProperty, value); }
    }

    public static readonly DependencyProperty MediaInfoProperty
      = DependencyProperty.Register("MediaInfo",
      typeof(Media),
      typeof(MediaInfoDisplay),
      new PropertyMetadata(new PropertyChangedCallback(OnMediaChanged)));

    // Handling the states
    private bool _isMouseOver = false;
    private bool _isDescriptionExpanded = false;
    private void GoToState(bool useTransitions)
    {
      if (_isMouseOver)
      {
        VisualStateManager.GoToState(this,
          "MouseOver", useTransitions);
      }
      else
      {
        VisualStateManager.GoToState(this,
          "Normal", useTransitions);
      }

      if (_isDescriptionExpanded)
      {
        VisualStateManager.GoToState(this,
          "DescriptionExpanded", useTransitions);
      }
      else
      {
        VisualStateManager.GoToState(this,
          "DescriptionNormal", useTransitions);
      }
    }

    // Handling the part
    private FrameworkElement _panelPart = null;
    private FrameworkElement DescriptionPanelPart
    {
      get { return _panelPart; }
      set
      {
        FrameworkElement _oldPart = _panelPart;
        if (_oldPart != null)
        {
          _oldPart.MouseEnter
            -= new MouseEventHandler(_panelPart_MouseEnter);
          _oldPart.MouseLeave
            -= new MouseEventHandler(_panelPart_MouseLeave);
          _oldPart.MouseLeftButtonDown
            -= new MouseButtonEventHandler(
              _panelPart_MouseLeftButtonDown);
        }
    
        _panelPart = value;
        if (_panelPart != null)
        {
          _panelPart.MouseEnter
            += new MouseEventHandler(_panelPart_MouseEnter);
          _panelPart.MouseLeave
            += new MouseEventHandler(_panelPart_MouseLeave);
          _panelPart.MouseLeftButtonDown
            += new MouseButtonEventHandler(
              _panelPart_MouseLeftButtonDown);
        }
      }
    }

    // Event handlers
    void _panelPart_MouseEnter(object sender,
      MouseEventArgs e)
    {
      _isMouseOver = true;
      GoToState(true);
    }
    void _panelPart_MouseLeave(object sender,
      MouseEventArgs e)
    {
      _isMouseOver = false;
      GoToState(true);
    }
    void _panelPart_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      _isDescriptionExpanded = !_isDescriptionExpanded;
      GoToState(true);
    }

    // Applying the template
    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      DescriptionPanelPart
        = (FrameworkElement) GetTemplateChild("DescriptionPanel");
      GoToState(false);
    }

  }
}
