﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;
using System.Linq;
using MediaInfo;
using System.Windows.Resources;
using System.Threading;
using System.Json;

namespace Thumbnails.Data
{
  public class MediaFileLoadedEventArgs : EventArgs
  {
    public IEnumerable<MediaEx> Collection
    {
      get;
      set;
    }
    public Exception Error
    {
      get;
      set;
    }
  }

  public class AnyProgressChangedEventArgs : EventArgs
  {
    public string Message
    {
      get;
      set;
    }
  }
  public class NewMediaFoundEventArgs : EventArgs
  {
    public MediaEx MediaInfo
    {
      get;
      set;
    }
  }


  public class MediaInformationFile
  {
    public const string MEDIA_FILE_NAME = "media.xml";
    public const string MEDIA_PATH = "MediaFiles";

    private const string FLICKR_API_KEY
      = "299b88d550e55e92796e58e15a686d19";

    private const string FLICKR_URI_BASE
      = "http://api.flickr.com/services/rest/"
      + "?method=flickr.photos.search"
      + "&api_key={0}&tags={1}&page={2}&per_page={3}"
      + "&format=json&tag_mode=all";

    private const string FLICKR_IMAGE_BASE
      = "http://farm{0}.static.flickr.com/{1}/{2}_{3}.jpg";

    private const string FLICKR_PREFIX = "jsonFlickrApi";

    private XDocument _document;

    public event EventHandler Loaded;
    public event DownloadProgressChangedEventHandler DownloadProgressChanged;

    private void OnMediaFileLoaded(IEnumerable<MediaEx> xmlMedias,
      Exception lastError)
    {
      if (Loaded != null)
      {
        MediaFileLoadedEventArgs args = new MediaFileLoadedEventArgs();
        args.Collection = xmlMedias;
        args.Error = lastError;
        Loaded(this, args);
      }
    }

    public event EventHandler NewMediaFound;
    public event EventHandler AnyProgressChanged;
    private void OnAnyProgressChanged(string message)
    {
      if (AnyProgressChanged != null)
      {
        AnyProgressChangedEventArgs args
          = new AnyProgressChangedEventArgs();
        args.Message = message;
        AnyProgressChanged(this, args);
      }
    }

    private SynchronizationContext _context;    
    
    public void LoadMediaFile()
    {
      WebClient client = new WebClient();
      client.DownloadStringCompleted
        += new DownloadStringCompletedEventHandler(
          client_DownloadStringCompleted);
      client.DownloadStringAsync(new Uri(MEDIA_FILE_NAME, UriKind.Relative));
    }

    void client_DownloadStringCompleted(object sender,
      DownloadStringCompletedEventArgs e)
    {
      // Prepare data
      IEnumerable<MediaEx> output = null;
      Exception lastError = null;
      try
      {
        StringReader reader = new StringReader(e.Result);
        _document = XDocument.Load(reader);
        output = from xmlMediaInfo
          in _document.Descendants("media")
          select new MediaEx
          {
            MediaPath = MEDIA_PATH,
            MediaName = (string) xmlMediaInfo.Attribute("name"),
            Type = (MediaType) Enum.Parse(typeof(MediaType),
              (string) xmlMediaInfo.Attribute("type"), true),
            Description = (string) xmlMediaInfo.Attribute("description")
          };
      }
      catch (Exception ex)
      {
        // Failure, inform user
        if (ex.InnerException != null
          && ex.InnerException is WebException)
        {
          lastError = ex.InnerException;
        }
        else
        {
          lastError = ex;
        }
        output = null;
      }

      if (output == null)
      {
        OnMediaFileLoaded(output, lastError);
      }
      else
      {
        LoadZipFile(output);
      }
    }

    private void LoadZipFile(IEnumerable<MediaEx> mediaInfos)
    {
      WebClient client = new WebClient();
      client.OpenReadCompleted
        += new OpenReadCompletedEventHandler(
          client_OpenReadCompleted);
      client.DownloadProgressChanged
        += new DownloadProgressChangedEventHandler(
          client_DownloadProgressChanged);
      client.OpenReadAsync(new Uri(MEDIA_PATH + ".zip", UriKind.Relative),
        mediaInfos);
    }

    void client_DownloadProgressChanged(object sender,
      DownloadProgressChangedEventArgs e)
    {
      if (DownloadProgressChanged != null)
      {
        DownloadProgressChanged(sender, e);
      }
    }

    void client_OpenReadCompleted(object sender,
      OpenReadCompletedEventArgs e)
    {
      MediaExCollection output = new MediaExCollection();
      Exception lastError = null;
      try
      {
        StreamResourceInfo zipInfo = new StreamResourceInfo(e.Result, null);
        IEnumerable<MediaEx> mediaInfos = e.UserState as IEnumerable<MediaEx>;

        foreach (MediaEx mediaInfo in mediaInfos)
        {
          StreamResourceInfo streamInfo
            = Application.GetResourceStream(zipInfo, mediaInfo.FileUri);
          mediaInfo.MediaSource = streamInfo.Stream;
          output.Add(mediaInfo);
        }
      }
      catch (Exception ex)
      {
        // Failure, inform user
        if (ex.InnerException != null
          && ex.InnerException is WebException)
        {
          lastError = ex.InnerException;
        }
        else
        {
          lastError = ex;
        }
        output = null;
      }

      OnMediaFileLoaded(output, lastError);

      if (NewMediaFound != null)
      {
        SendFlickrRequests();
      }
    }

    private void SendFlickrRequests()
    {
      var flickrSearches
        = from xmlSearch
          in _document.Descendants("flickr-search")
          select new
          {
            Keywords = xmlSearch.Attribute("keywords").Value,
            PerPage 
              = Int32.Parse(xmlSearch.Attribute("per-page").Value),
            StartPage
              = Int32.Parse(xmlSearch.Attribute("start-page").Value)
          };
    
      _context = SynchronizationContext.Current;
    
      foreach (var flickrSearch in flickrSearches)
      {
        OnAnyProgressChanged("Starting loading Flickr searches");
        string uriString = string.Format(FLICKR_URI_BASE,
          FLICKR_API_KEY,
          flickrSearch.Keywords,
          flickrSearch.StartPage,
          flickrSearch.PerPage);
    
        Uri uri = new Uri(uriString, UriKind.Absolute);
        HttpWebRequest request 
          = WebRequest.Create(uri) as HttpWebRequest;
        request.Method = "GET";
        IAsyncResult result
          = request.BeginGetResponse(
            new AsyncCallback(ResponseCallback), request);
      }
    }

    private void ResponseCallback(IAsyncResult result)
    {
      HttpWebRequest request = result.AsyncState as HttpWebRequest;
      
      // Change here
      WebResponse response;
      try
      {
        response = request.EndGetResponse(result);
      }
      catch (Exception)
      {
        response = null;
      }
      _context.Post(ExtractResponse, response);
    }

    private void ExtractResponse(object state)
    {
      OnAnyProgressChanged("Flickr response received, decoding");
      HttpWebResponse response = state as HttpWebResponse;

      // Change here
      if (response == null)
      {
        OnAnyProgressChanged("Request failed");
        return;
      }

      Stream flickrStream = response.GetResponseStream();

      try
      {
        string jsonString;
        using (StreamReader responseReader = new StreamReader(flickrStream))
        {
          jsonString = responseReader.ReadToEnd();
          // Remove prefix and parenthesis
          jsonString = jsonString.Substring(FLICKR_PREFIX.Length); // Remove prefix
          jsonString = jsonString.Substring(1, jsonString.Length - 2); // Remove parenthesis
        }

        using (StringReader stringReader = new StringReader(jsonString))
        {
          JsonObject jsonResponse = JsonObject.Load(stringReader) as JsonObject;

          var photos = from photo in (JsonArray) jsonResponse["photos"]["photo"]
                       let castedPhoto = photo as JsonObject
                       select new
                       {
                         Farm = castedPhoto["farm"],
                         Server = (string) castedPhoto["server"],
                         Id = (string) castedPhoto["id"],
                         Secret = (string) castedPhoto["secret"],
                         Title = (string) castedPhoto["title"]
                       };

          foreach (var photo in photos)
          {
            char[] trimChar = new char[] { '"' };
            string photoUriString = string.Format(FLICKR_IMAGE_BASE,
              photo.Farm,
              photo.Server,
              photo.Id,
              photo.Secret);

            Uri photoUri = new Uri(photoUriString, UriKind.Absolute);

            WebClient clientPhoto = new WebClient();
            clientPhoto.OpenReadCompleted
              += new OpenReadCompletedEventHandler(
                clientPhoto_OpenReadCompleted);
            clientPhoto.OpenReadAsync(photoUri, photo.Title);
          }
        }
      }
      catch (Exception)
      {
        OnAnyProgressChanged("Error with Flickr search");
      }
    }

    void clientPhoto_OpenReadCompleted(object sender,
      OpenReadCompletedEventArgs e)
    {
      try
      {
        string title = e.UserState.ToString();
        MediaEx mediaInfo = new MediaEx();
        mediaInfo.Description = title + " (Flickr search result)";
        mediaInfo.MediaSource = e.Result;
        NewMediaFoundEventArgs args = new NewMediaFoundEventArgs();
        args.MediaInfo = mediaInfo;
        NewMediaFound(this, args);
      }
      catch (Exception)
      {
        OnAnyProgressChanged("Error with Flickr image");
      }
    }

  }
}
